@@ -1,7 +1,7 @@
 use crate::lang::SgLang;
 use crate::print::ColorArg;
 use ansi_term::Style;
-use ast_grep_core::Pattern;
+use ast_grep_core::{language::TSLanguage, matcher::PatternNode, meta_var::MetaVariable, Pattern};
 use ast_grep_language::Language;
 use clap::ValueEnum;
 use tree_sitter as ts;
@@ -18,24 +18,28 @@ pub enum DebugFormat {
   Sexp,
 }
 impl DebugFormat {
-  pub fn debug_query(&self, pattern: &str, lang: SgLang, color: ColorArg) {
+  pub fn debug_query(&self, src: &str, pattern: &Pattern<SgLang>, lang: SgLang, color: ColorArg) {
     let colored = color.should_use_color();
     match self {
       DebugFormat::Pattern => {
-        let pattern = Pattern::try_new(pattern, lang).expect("pattern must be validated in run");
-        println!("Debug Pattern:\n{:?}", pattern);
+        let lang = lang.get_ts_language();
+        let mut ret = String::new();
+        let fmt = DumpFmt::named(colored);
+        dump_pattern(&pattern.node, &lang, &fmt, 0, &mut ret)
+          .expect("unexpected error in writing string");
+        println!("Debug Pattern:\n{}", ret);
       }
       DebugFormat::Sexp => {
-        let root = lang.ast_grep(pattern);
+        let root = lang.ast_grep(src);
         println!("Debug Sexp:\n{}", root.root().to_sexp());
       }
       DebugFormat::Ast => {
-        let root = lang.ast_grep(pattern);
+        let root = lang.ast_grep(src);
         let dumped = dump_node(root.root().get_ts_node());
         println!("Debug AST:\n{}", dumped.ast(colored));
       }
       DebugFormat::Cst => {
-        let root = lang.ast_grep(pattern);
+        let root = lang.ast_grep(src);
         let dumped = dump_node(root.root().get_ts_node());
         println!("Debug CST:\n{}", dumped.cst(colored));
       }
@@ -43,6 +47,51 @@ impl DebugFormat {
   }
 }
 
+fn dump_pattern(
+  pattern: &PatternNode,
+  lang: &TSLanguage,
+  style: &DumpFmt,
+  indent: usize,
+  ret: &mut String,
+) -> FmtResult {
+  write!(ret, "{}", "  ".repeat(indent))?;
+  match pattern {
+    PatternNode::MetaVar { meta_var } => {
+      let meta_var = match meta_var {
+        MetaVariable::Capture(name, _) => format!("${name}"),
+        MetaVariable::MultiCapture(name) => format!("$$${name}"),
+        MetaVariable::Multiple => "$$$".to_string(),
+        MetaVariable::Dropped(_) => "$_".to_string(),
+      };
+      let meta_var = style.kind_style.paint(meta_var);
+      writeln!(ret, "{} {meta_var}", style.field_style.paint("MetaVar"))?;
+    }
+    PatternNode::Terminal {
+      text,
+      kind_id,
+      is_named,
+    } => {
+      let lang = lang.get_ts_language();
+      if *is_named {
+        let kind = lang.node_kind_for_id(*kind_id).unwrap();
+        let kind = style.kind_style.paint(format!("{kind}"));
+        writeln!(ret, "{kind} {text}")?;
+      } else {
+        writeln!(ret, "{text}")?;
+      }
+    }
+    PatternNode::Internal { kind_id, children } => {
+      let kind = lang.node_kind_for_id(*kind_id).unwrap();
+      let kind = style.kind_style.paint(kind);
+      writeln!(ret, "{kind}")?;
+      for child in children {
+        dump_pattern(child, lang, style, indent + 1, ret)?;
+      }
+    }
+  }
+  Ok(())
+}
+
 pub struct DumpNode {
   field: Option<String>,
   kind: String,
@@ -77,7 +126,6 @@ impl DumpFmt {
   }
 }
 
-// TODO: add colorized output
 use std::fmt::{Result as FmtResult, Write};
 impl DumpNode {
   pub fn ast(&self, colored: bool) -> String {
@@ -178,3 +226,39 @@ fn dump_nodes(cursor: &mut ts::TreeCursor, target: &mut Vec<DumpNode>) {
     }
   }
 }
+
+#[cfg(test)]
+mod test {
+  use super::*;
+  use ast_grep_language::{TypeScript, C};
+  const DUMPED: &str = r#"
+program (0,0)-(0,11)
+  variable_declaration (0,0)-(0,11)
+    variable_declarator (0,4)-(0,11)
+      name: identifier (0,4)-(0,5)
+      value: number (0,8)-(0,11)"#;
+  #[test]
+  fn test_dump_node() {
+    let lang = SgLang::Builtin(TypeScript.into());
+    let root = lang.ast_grep("var a = 123");
+    let dumped = dump_node(root.root().get_ts_node());
+    assert_eq!(DUMPED.trim(), dumped.ast(false).trim());
+  }
+
+  const MISSING: &str = r#"
+translation_unit (0,0)-(0,9)
+  declaration (0,0)-(0,9)
+    type: primitive_type (0,0)-(0,3)
+    declarator: init_declarator (0,4)-(0,9)
+      declarator: identifier (0,4)-(0,5)
+      = (0,6)-(0,7)
+      value: number_literal (0,8)-(0,9)
+    MISSING ; (0,9)-(0,9)"#;
+  #[test]
+  fn test_missing_node() {
+    let lang = SgLang::Builtin(C.into());
+    let root = lang.ast_grep("int a = 1");
+    let dumped = dump_node(root.root().get_ts_node());
+    assert_eq!(MISSING.trim(), dumped.cst(false).trim());
+  }
+}
