@@ -1,4 +1,5 @@
 mod custom_lang;
+mod injection;
 mod lang_globs;
 
 use anyhow::Result;
@@ -18,9 +19,10 @@ use std::path::{Path, PathBuf};
 use std::str::FromStr;
 
 pub use custom_lang::CustomLang;
+pub use injection::SerializableInjection;
 pub use lang_globs::LanguageGlobs;
 
-#[derive(Copy, Clone, PartialEq, Eq, Serialize, Deserialize)]
+#[derive(Copy, Clone, PartialEq, Eq, Serialize, Deserialize, Hash)]
 #[serde(untagged)]
 pub enum SgLang {
   // inlined support lang expando char
@@ -51,6 +53,10 @@ impl SgLang {
     Ok(())
   }
 
+  pub fn register_injections(injections: Vec<SerializableInjection>) -> Result<()> {
+    unsafe { injection::register_injetables(injections) }
+  }
+
   pub fn all_langs() -> Vec<Self> {
     let builtin = SupportLang::all_langs().iter().copied().map(Self::Builtin);
     let customs = DynamicLang::all_langs().into_iter().map(Self::Custom);
@@ -180,17 +186,11 @@ impl Language for SgLang {
   }
 
   fn injectable_languages(&self) -> Option<&'static [&'static str]> {
-    match self {
-      Builtin(b) => b.injectable_languages(),
-      Custom(c) => c.injectable_languages(),
-    }
+    injection::injectable_languages(*self)
   }
 
   fn extract_injections<D: Doc>(&self, root: Node<D>) -> HashMap<String, Vec<TSRange>> {
-    match self {
-      Builtin(b) => b.extract_injections(root),
-      Custom(c) => c.extract_injections(root),
-    }
+    injection::extract_injections(root)
   }
 }
 
